/*
 * Debug console input/output for MIPS microcontrollers.
 * Send a byte to the UART transmitter, with interrupts disabled.
 */

 /*
 * External UART registers.
 */


#include "simple_time.h"
#include "uart.h"
#include "nvcom_01.h"

#define MC_R(a)		*(volatile unsigned*)(0xB82F0000 + (a))

#define MC_RBR                MC_R (0x3000)        /*    */
#define MC_THR                MC_R (0x3000)        /*    */
#define MC_IER                MC_R (0x3004)        /*    */
#define MC_IIR                MC_R (0x3008)        /*    */
#define MC_FCR                MC_R (0x3008)        /*   FIFO */
#define MC_LCR                MC_R (0x300C)        /*    */
#define MC_MCR                MC_R (0x3010)        /*    */
#define MC_LSR                MC_R (0x3014)        /*    */
#define MC_MSR                MC_R (0x3018)        /*    */
#define MC_SPR                MC_R (0x301C)        /*  Scratch Pad */
#define MC_DLL		MC_R (0x3000)	/*    */
#define MC_DLM		MC_R (0x3004)	/*    */
#define MC_SCLR		MC_R (0x3014)	/*   (scaler) */

/*--------------------------------------
 * UART.
 */
/*
 * Line control register
 */
#define MC_LCR_5BITS            0x00    /* character length: 5 bits */
#define MC_LCR_6BITS            0x01    /* character length: 6 bits */
#define MC_LCR_7BITS            0x02    /* character length: 7 bits */
#define MC_LCR_8BITS            0x03    /* character length: 8 bits */

#define MC_LCR_STOPB            0x04    /* use 2 stop bits */
#define MC_LCR_PENAB            0x08    /* parity enable */
#define MC_LCR_PEVEN            0x10    /* even parity */
#define MC_LCR_PFORCE           0x20    /* force parity */
#define MC_LCR_SBREAK           0x40    /* break control */
#define MC_LCR_DLAB             0x80    /* divisor latch access bit */

/*
 * FIFO control register
 */
#define MC_FCR_ENABLE                0x01        /* enable FIFO */
#define MC_FCR_RCV_RST                0x02        /* clear receive FIFO */
#define MC_FCR_XMT_RST                0x04        /* clear transmit FIFO */

#define MC_FCR_TRIGGER_1        0x00        /* receive FIFO level: 1/4 byte */
#define MC_FCR_TRIGGER_4        0x40        /* receive FIFO level: 4/16 bytes */
#define MC_FCR_TRIGGER_8        0x80        /* receive FIFO level: 8/56 bytes */
#define MC_FCR_TRIGGER_14        0xc0        /* receive FIFO level: 14/60 bytes */

/*
 * Line status register
 */
#define MC_LSR_RXRDY                0x01        /* receiver ready */
#define MC_LSR_OE                0x02        /* overrun error */
#define MC_LSR_PE                0x04        /* parity error */
#define MC_LSR_FE                0x08        /* framing error */
#define MC_LSR_BI                0x10        /* break interrupt */
#define MC_LSR_TXRDY                0x20        /* transmitter holding register empty */
#define MC_LSR_TEMT                0x40        /* transmitter empty */
#define MC_LSR_FIFOE                0x80        /* error in receive FIFO */

/*
 * Interrupt enable register
 */
#define MC_IER_ERXRDY                0x01        /* enable receive data/timeout intr */
#define MC_IER_ETXRDY                0x02        /* enable transmitter interrupts */
#define MC_IER_ERLS                0x04        /* enable receive line status intr */
#define MC_IER_EMSC                0x08        /* enable modem status interrupts */

/*
 * Interrupt identification register
 */
#define MC_IIR_NOPEND                0x01        /* no interrupt pending */
#define MC_IIR_IMASK                0x0e        /* interrupt type mask */
#define MC_IIR_FENAB		0xc0	/* set if FIFOs are enabled */

#define MC_IIR_RLS		0x06	/* receiver line status */
#define MC_IIR_RXRDY		0x04	/* receiver data available */
#define MC_IIR_RXTOUT		0x0c	/* character timeout indication */
#define MC_IIR_TXRDY		0x02	/* transmitter holding reg empty */
#define MC_IIR_MLSC		0x00	/* modem status */

/*
 * Modem control register
 */
#define MC_MCR_DTR		0x01	/* control DTR output */
#define MC_MCR_RTS		0x02	/* control RTS output */
#define MC_MCR_OUT1		0x04	/* control OUT1 output */
#define MC_MCR_OUT2		0x08	/* control OUT2 output, used as
					 * global interrupt enable in PCs */
#define MC_MCR_LOOPBACK		0x10	/* set local loopback mode */

/*
 * Modem status register
 */
#define MC_MSR_DCTS		0x01	/* CTS changed */
#define MC_MSR_DDSR		0x02	/* DSR changed */
#define MC_MSR_TERI		0x04	/* RI changed from 0 to 1 */
#define MC_MSR_DDCD		0x08	/* DCD changed */
#define MC_MSR_CTS		0x10	/* CTS input */
#define MC_MSR_DSR		0x20	/* DSR input */
#define MC_MSR_RI		0x40	/* RI input */
#define MC_MSR_DCD		0x80	/* DCD input */

/*
 * Compute the 16-bit baud rate divisor, given
 * the oscillator frequency and baud rate.
 * Round to the nearest integer.
 */
#define MC_DL_BAUD(fr,bd)	(((fr)/8 + (bd)) / (bd) / 2)

/*
 * Baudrate generator source - CLK
 */
#define setup_baud_rate(khz, baud)	{ \
	unsigned divisor = MC_DL_BAUD (khz * 1000, baud);		\
	MC_LCR = MC_LCR_8BITS | MC_LCR_DLAB;			\
	MC_DLM = divisor >> 8;					\
	MC_DLL = divisor;					\
	MC_LCR = MC_LCR_8BITS;					\
}

/**
 *   
 * @return -    Hz
 */
unsigned get_cpu_clock(void)
{
    return((CLOCK)*(CR_PLL&0xff));
};

void
uart_config(unsigned int khz, unsigned long baud)
{
	/* Setup baud rate generator. */
//	setup_baud_rate (khz, baud);

	//unsigned divisor = MC_DL_BAUD (khz * 1000, baud);
        unsigned divisor = MC_DL_BAUD (get_cpu_clock(), baud);


	MC_LCR = MC_LCR_8BITS | MC_LCR_DLAB;
	MC_DLM = divisor >> 8;
	MC_DLL = divisor;
	MC_LCR = MC_LCR_8BITS;
	MC_SCLR = 0;
	MC_SPR = 0;
	MC_IER = 0;
	MC_MSR = 0;
	MC_MCR = MC_MCR_DTR | MC_MCR_RTS | MC_MCR_OUT2;
	MC_FCR = MC_FCR_RCV_RST | MC_FCR_XMT_RST | MC_FCR_ENABLE;

	/* Clear pending status, data and irq. */

	(void) MC_LSR;
	(void) MC_MSR;
	(void) MC_RBR;
	(void) MC_IIR;

}

void uart_putchar (short c)
{
        int x;

        /* Wait for transmitter holding register empty. */
        while (! (MC_LSR & MC_LSR_TXRDY))
                continue;
again:
        /* Send byte. */
        MC_THR = c;

        /* Wait for transmitter holding register empty. */
        while (! (MC_LSR & MC_LSR_TXRDY))
                continue;

        if (c == '\n') {
                c = '\r';
                goto again;
        }
}

/*
 * Wait for the byte to be received and return it.
 */
unsigned short uart_getchar (void)
{
        unsigned char c;
        int x;

        for (;;){
                /* Wait until receive data available. */
                if (! (MC_LSR & MC_LSR_RXRDY)) {
                        continue;
                }
                c = MC_RBR;
                break;
        }
        return c;
}

